SHADER version 1
@OpenGL2.Vertex
#version 400
#define MAX_INSTANCES 256

//Uniforms
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform mat4 projectioncameramatrix;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_tangent;
in vec3 vertex_binormal;

//Outputs
out vec4 ex_vertexposition;
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;

void main()
{
	//entity matrix
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_ = entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	//model position
	ex_vertexposition = entitymatrix_ * vec4(vertex_position, 1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;

	//normals
	mat3 nmat = mat3(entitymatrix_);
	ex_normal = normalize(nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat * vertex_binormal);

	//texture coordinates
	ex_texcoords0 = vertex_texcoords0;
}
@OpenGLES2.Vertex

@OpenGLES2.Fragment

@OpenGL4.Vertex
#version 400
#define MAX_INSTANCES 256

//Uniforms
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform mat4 projectioncameramatrix;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_tangent;
in vec3 vertex_binormal;

//Outputs
out vec4 ex_vertexposition;
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;

void main()
{
	//entity matrix
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_ = entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	//model position
	ex_vertexposition = entitymatrix_ * vec4(vertex_position, 1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;

	//normals
	mat3 nmat = mat3(entitymatrix_);
	ex_normal = normalize(nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat * vertex_binormal);

	//texture coordinates
	ex_texcoords0 = vertex_texcoords0;
}
@OpenGL4.Fragment
//from https://www.shadertoy.com/view/Msf3WH

#version 400

//Variables
float specular = 0.3;
float specular2 = 0.2;
float tex_size = 1.0;
float tex_size2 = 2.0;
float mask_size = 1.0;

//Uniforms
uniform sampler2D texture0;
uniform sampler2D texture1;
uniform sampler2D texture6;
uniform sampler2D texture7;

//Inputs
in vec4 ex_vertexposition;
in vec2 ex_texcoords0;
in vec3 ex_normal;
in vec3 ex_tangent;
in vec3 ex_binormal;

//Outputs
out vec4 fragData0;
out vec4 fragData1;
out vec4 fragData2;

vec2 hash( vec2 p ) // replace this by something better
{
	p = vec2( dot(p,vec2(127.1,311.7)), dot(p,vec2(269.5,183.3)) );
	return -1.0 + 2.0*fract(sin(p)*43758.5453123);
}

float noise( in vec2 p )
{
    const float K1 = 0.366025404; // (sqrt(3)-1)/2;
    const float K2 = 0.211324865; // (3-sqrt(3))/6;

	vec2  i = floor( p + (p.x+p.y)*K1 );
    vec2  a = p - i + (i.x+i.y)*K2;
    float m = step(a.y,a.x); 
    vec2  o = vec2(m,1.0-m);
    vec2  b = a - o + K2;
	vec2  c = a - 1.0 + 2.0*K2;
    vec3  h = max( 0.5-vec3(dot(a,a), dot(b,b), dot(c,c) ), 0.0 );
	vec3  n = h*h*h*h*vec3( dot(a,hash(i+0.0)), dot(b,hash(i+o)), dot(c,hash(i+1.0)));
    return dot( n, vec3(70.0) );
}


void main() {

	//Noise
	float f;
    vec2 uv = ex_texcoords0 * mask_size;
	uv.x *= 1.0;
	uv.y *= 1.0;
	uv *= 5.0;
    mat2 m = mat2(1.6,  1.2, -1.2,  1.6);
    f  = 0.5000*noise(uv); uv = m*uv;
	f += 0.2500*noise(uv); uv = m*uv;
	f += 0.1250*noise(uv); uv = m*uv;
	f += 0.0625*noise(uv); uv = m*uv;
	f = 0.5 + 1.0*f;	
	float contrast = clamp(dot(vec3(f),vec3(1.0))-1.0,0.0,1.0);
	f *= contrast;

	//Diffuse
	vec4 outcolor = texture(texture0,ex_texcoords0*tex_size);
	vec4 out2 = texture(texture6,ex_texcoords0*tex_size2);
	outcolor = mix(outcolor,out2,f);
	
	//Normal
	vec3 normal = ex_normal;
	normal = normalize(texture(texture1,ex_texcoords0*tex_size).xyz * 2.0 - 1.0);
	vec3 normal2 = normalize(texture(texture7,ex_texcoords0*tex_size2).xyz * 2.0 - 1.0);
	normal = mix(normal,normal2,f);
	normal = ex_tangent * normal.x + ex_binormal * normal.y + ex_normal * normal.z;	
	normal = normalize(normal);

	//Output
	fragData0 = outcolor;
	
	//Normals
	fragData1 = vec4(normalize(normal)*0.5+0.5,fragData0.a);
	fragData1.a = 1/255.0;
	
	fragData2 = mix(vec4(0,0,0,specular),vec4(0,0,0,specular2),f);
}
